import { isAsyncIterable } from './utils/iterator.js';
export function filter(fn) {
    return function filterFn(data) {
        if (isAsyncIterable(data) || data instanceof Promise) {
            return {
                async *[Symbol.asyncIterator]() {
                    let stream = data instanceof Promise ? await data : data;
                    let i = 0;
                    for await (let datum of stream) {
                        // Ignore values that do not meet the criteria
                        if (!fn(datum, i++))
                            continue;
                        yield datum;
                    }
                },
            };
        }
        return {
            *[Symbol.iterator]() {
                let i = 0;
                for (let datum of data) {
                    // Ignore values that do not meet the criteria
                    if (!fn(datum, i++))
                        continue;
                    yield datum;
                }
            },
        };
    };
}
