import { isAsyncIterable } from './utils/iterator.js';
export function slice(begin = 0, end = Infinity) {
    return function sliceFn(data) {
        if (isAsyncIterable(data) || data instanceof Promise) {
            return {
                async *[Symbol.asyncIterator]() {
                    let stream = data instanceof Promise ? await data : data;
                    let iterator = stream;
                    let local_begin = begin;
                    let local_end = end - local_begin;
                    // Skip the first X values
                    while (local_begin-- > 0)
                        iterator.next();
                    // Loop through the remaining items until the end is reached
                    for await (let datum of iterator) {
                        yield datum;
                        if (--local_end < 0)
                            return;
                    }
                },
            };
        }
        return {
            *[Symbol.iterator]() {
                let iterator = Array.isArray(data)
                    ? data[Symbol.iterator]()
                    : data;
                let local_begin = begin;
                let local_end = end - local_begin;
                // Skip the first X values
                while (local_begin-- > 0)
                    iterator.next();
                // Loop through the remaining items until the end is reached
                for (let datum of iterator) {
                    yield datum;
                    if (--local_end < 0)
                        return;
                }
            },
        };
    };
}
